#!/usr/bin/ruby

#
# AS - Agregador Simple - apanha duma lista as url dos feeds e cria um corpus com esse feed
#

Dir.chdir(File.dirname(__FILE__))

require 'rubygems'  
require 'feed-normalizer'
require 'lib/peach.rb'
require 'lib/reco_fun'
require "date"


SRC_DIRECTORY = "src" # Where is the directory with the files
CORPUS_DIRECTORY = "corpus" # Where is the directory for the corpus
DATA_DIRECTORY = "data" # Where is the directory for the internal program data (like visited urls)
LOG_DIRECTORY = "log" # Where is the directory for save logs about error or success

# Caracteres especiais
CONVERCOM_HTML = [[/&nbsp;/," "],[/&Agrave;/,"À"],[/&Aacute;/,"Á"],[/&Acirc;/,"Â"],[/&Atilde;/,"Ã"],[/&Ograve;/,"Ô"],[/&Ccedil;/,"Ç"],[/&Eacute;/,"É"],[/&Ecirc;/,"Ê"],[/&Iacute;/,"Í"],[/&Ntilde;/,"Ñ"],[/&Oacute;/,"Ó"],[/&Ocirc;/,"Ô"],[/&Otilde;/,"Õ"],[/&Uacute;/,"Ú"],[/&agrave;/,"á	"],[/&aacute;/,"á"],[/&acirc;/,"â"],[/&atilde;/,"ã"],[/&ccedil;/,"ç"],[/&eacute;/,"é"],[/&ecirc;/,"ê"],[/&iacute;/,"í"],[/&ntilde;/,"ñ"],[/&ograve;/,""],[/&oacute;/,"ó"],[/&ocirc;/,"ô"],[/&otilde;/,"õ"],[/&uacute;/,"ú"],[/&uuml;/,"ü"],[/&ordm;/,"º"],[/&iquest;/,"¿"],[/&laquo;/,"«"],[/&raquo;/,"»"],[/&ldquo;/,'"'],[/&rdquo;/,'"'],[/&quot;/,'"'],[/&lsquo;/,'"'],[/&rsquo;/,'"'],["","-"],["",'"'],["",'"'],["&#039;",'"']]

# Limpa a cadea entrante e devólvea sem os tag ("<calqueracousa">) e convertindo os &ntilde; e similares
def limpar_html(cadea)	
	# limpamos os tags html
	cadea = cadea.gsub(/<\/?[^>]*>/, "")
	CONVERCOM_HTML.each do |c|
		cadea = cadea.gsub(c[0], c[1])
	end
	cadea
end

# Clean white spaces (tabs and newlines) before and after last letters
def limpar_brancos(cadea)
	if cadea == nil
		return cadea
	end
	while cadea[0..0] == " " || cadea[0..0] == "\t"
		cadea = cadea[1..-1]
	end
	while cadea[-1..-1] == " " || cadea[-1..-1] == "\n" || cadea[-1..-1] == "\t"
		cadea = cadea[0..-2]
	end
	cadea
end	


############## SOURCE CLASS
# Represents the newspaper, web ot whatever information source
class Source

	# Build method
	def initialize(name,rss_url)
		@name = name
		@rss_url = rss_url.chop
		@corpus_file = File.new("#{CORPUS_DIRECTORY}/#{@name}_corpus.txt","a+")
		@visited_urls_file = File.new("#{DATA_DIRECTORY}/#{@name}_visited_urls.txt","a+")
		@visited_urls = Array.new
		@visited_urls_file.each { |line| @visited_urls << line.chomp }
		@updated = false
		@posts_updated = 0

		# Call methods
		begin
			aggregate
		rescue => e
			LOG_ERROR.puts "erro agregando o feed de " + @name + " - " + @rss_url
		end
		close
	end

	# Aggregate new items
	def aggregate
		# Get rss
		rss = get_rss @rss_url	
		#Read entries  
		rss.entries.each do |entry|
			if (not @visited_urls.include? entry.urls.first) && (limpar_brancos(limpar_html(entry.content)) != "") && (reco(limpar_brancos(limpar_html(entry.content))) == "gl")
				@corpus_file.puts "<item>"
				@corpus_file.puts "\t<titulo>#{entry.title}</titulo>"
				@corpus_file.puts "\t<contido>#{limpar_brancos(limpar_html(entry.content))}</contido>"
				@corpus_file.puts "\t<url>#{entry.urls.first}</url>"
				@corpus_file.puts "\t<data>#{Time.now}</data>"
				@corpus_file.puts "</item>"
				@visited_urls_file.puts entry.urls.first
				LOG.print "*"
				print "*" # Saida por ecra
				@updated = true
				@posts_updated += 1
			end
		end
		LOG.puts "\nActualizado " + @name
		puts "\nActualizado " + @name # Saida por ecra
	end

	# Task for close files
	def close
		@corpus_file.close		
		@visited_urls_file.close
	end

	def updated?
		@updated
	end

	def posts_updated
		@post_updated
	end

	private

	# Get the rss in an xml format
	def get_rss(feed_url)
		begin
			feed_content = open(feed_url)
		rescue => e
			log_error.print "Erro abrindo a url #{feed_url}"
		rescue Timeout::Error => e
			log_error.print "Timeour abrindo a url #{feed_url}"
		end
		FeedNormalizer::FeedNormalizer.parse feed_content  
	end
end




############## MAIN


#Precode
if not File::exist? CORPUS_DIRECTORY
	Dir.mkdir(CORPUS_DIRECTORY)
end
if not File::exist? DATA_DIRECTORY
	Dir.mkdir(DATA_DIRECTORY)
end
if not File::exist? SRC_DIRECTORY
	Dir.mkdir(SRC_DIRECTORY)
end
if not File::exist? SRC_DIRECTORY + "/sources.txt"
	puts "Creates a file with the names and RSS sources(each line have: name + tab + rss_url), called sources.txt in the folder " + SRC_DIRECTORY
	exit
end
if not File::exist? LOG_DIRECTORY
	Dir.mkdir(LOG_DIRECTORY)
end

LOG = File.new("#{LOG_DIRECTORY}/log_#{Date.today.to_s}.txt","a")
LOG.puts "   #{Time.now}   ".center(90,"#")
LOG_ERROR = File.new("#{LOG_DIRECTORY}/log_error_#{Date.today.to_s}.txt","a")
LOG_ERROR.puts "   #{Time.now}   ".center(90,"#")


sources = File.open(SRC_DIRECTORY + "/sources.txt")

ti = Time.now
num_sources = 0
num_updated = 0
num_posts_updated = 0
sources.each do |line|
	name = line.split("\t")[0].gsub(" ","").gsub("/","_")
	url = line.split("\t")[1]
	if name != nil && url != nil && name != "" && url != ""
		begin
			source = Source.new(name,url)
			if source.updated?
				num_updated += 1
			end
			num_sources += 1
			num_posts_updated = source.posts_updated
		rescue => e
			LOG_ERROR.puts "Erro en " + name + " - " + $!
		end
	end
end
puts "Time working: #{(Time.now - ti).to_i} seconds"
puts "Total sources: #{num_sources}   Total sourced updated: #{num_updated}    Total posts added:#{num_posts_updated}"

